﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Ogc.Filter.Expressions
{

    public abstract class BinaryOperator : IExpression
    {
        IExpression expra;
        IExpression exprb;

        protected BinaryOperator(IExpression expra, IExpression exprb)
        {
            this.expra = expra;
            this.exprb = exprb;
        }

        protected abstract object Evaluate(object a, object b);

        #region IExpression 成员

        public object Evaluate(IPropertyGetter getter)
        {
            return Evaluate(expra.Evaluate(getter), exprb.Evaluate(getter));
        }

        #endregion
    }

    public class Add : BinaryOperator
    {
        public Add(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override object Evaluate(object a, object b)
        {
            if (a is string && b is string)
                return a.ToString() + b.ToString();
            if (a is double && b is double)
                return (double)a + (double)b;
            throw new NotImplementedException();
        }
    }

    public class Sub : BinaryOperator
    {
        public Sub(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override object Evaluate(object a, object b)
        {
            if (a is double && b is double)
                return (double)a - (double)b;
            throw new NotImplementedException();
        }
    }

    public class Mul : BinaryOperator
    {
        public Mul(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override object Evaluate(object a, object b)
        {
            if (a is double && b is double)
                return (double)a * (double)b;
            throw new NotImplementedException();
        }
    }

    public class Div : BinaryOperator
    {
        public Div(IExpression expra, IExpression exprb)
            : base(expra, exprb)
        {
        }

        protected override object Evaluate(object a, object b)
        {
            if (a is double && b is double)
                return (double)a / (double)b;
            throw new NotImplementedException();
        }
    }
}
