﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Xml;
using Ogc.Filter.LogicOps;
using Ogc.Filter.SpatialOps;
using Ogc.Filter.ComparisonOps;
using Ogc.Filter.Expressions;

namespace Ogc.Filter
{
    public class FilterBuilder
    {
        readonly static string[] OpNames = new string[] {
            "And"
            ,"Or"
            ,"Not"
            ,"Equals"  
            ,"Disjoint"
            ,"Touches" 
            ,"Within"  
            ,"Overlaps"
            ,"Crosses"
            ,"Intersect"
            ,"Contains"
            ,"DWithin" 
            ,"Beyond"  
            ,"BBOX"
            ,"PropertyIsEqualTo"
            ,"PropertyIsNotEqualTo"
            ,"PropertyIsLessThan"
            ,"PropertyIsGreaterThan"
            ,"PropertyIsLessThanOrEqualTo"
            ,"PropertyIsGreaterThanOrEqualTo"
            ,"PropertyIsLike"
            ,"PropertyIsNull"
            ,"PropertyIsBetween"
        };

        public IFilter BuildFilter(XmlNode xmlnode)
        {
            foreach (string i in OpNames)
            {
                XmlNode n = xmlnode.SelectSingleNode(i);
                if (n != null)
                    return ((IFilterFactory)Activator.CreateInstance(Type.GetType("Ogc.Filter." + i + "Factory"))).FromXmlNode(n, this);
            }
            throw new NotImplementedException();
        }

        public IExpression BuildExpression(XmlNode xmlnode)
        {
            return ((IExpressionFactory)Activator.CreateInstance(Type.GetType("Ogc.Filter." + xmlnode.Name + "Factory")))
                .FromXmlNode(xmlnode, this);
        }

        public object BuildValue(XmlNode xmlNode)
        {
            string s = xmlNode.InnerText;
            double d;
            if (double.TryParse(s, out d))
                return d;
            return s;
        }
    }

    public interface IFilterFactory
    {
        IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder);
    }

    public interface IExpressionFactory
    {
        IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder);
    }

    #region LogicOpFactory

    class AndFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new And(builder.BuildFilter(xmlnode.SelectSingleNode("*[1]"))
                , builder.BuildFilter(xmlnode.SelectSingleNode("*[2]")));
        }

        #endregion
    }

    class OrFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new Or(builder.BuildFilter(xmlnode.SelectSingleNode("*[1]"))
                , builder.BuildFilter(xmlnode.SelectSingleNode("*[2]")));
        }

        #endregion
    }

    class NotFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new Not(builder.BuildFilter(xmlnode.SelectSingleNode("*[1]")));
        }

        #endregion
    }
    #endregion

    #region SpatialOpFactory
    class EqualsFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            string geoname = xmlnode.SelectSingleNode("PropertyName").InnerText;

            XmlNode n = xmlnode.SelectSingleNode("Envelope");

            throw new NotImplementedException();
        }

        #endregion
    }
    class DisjointFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class TouchesFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class WithinFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class OverlapsFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class CrossesFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class IntersectFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class ContainsFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class DWithinFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class BeyondFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    class BBOXFactory : IFilterFactory
    {
        #region ITesterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }
    #endregion

    #region ComparisonOpFactory

    public abstract class BinaryComparisonOpFactory : IFilterFactory
    {
        protected abstract BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb);

        #region IFilterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            BinaryComparisonOp ret = CreateBinaryComparisonOp(builder.BuildExpression(xmlnode.SelectSingleNode("*[1]"))
                , builder.BuildExpression(xmlnode.SelectSingleNode("*[2]")));
            ret.MatchCase = false;
            return ret;
        }

        #endregion
    }

    public class PropertyIsEqualToFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsEqualTo(expra, exprb);
        }
    }

    public class PropertyIsNotEqualToFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsNotEqualTo(expra, exprb);
        }
    }

    public class PropertyIsLessThanFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsLessThan(expra, exprb);
        }
    }

    public class PropertyIsGreaterThanFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsGreaterThan(expra, exprb);
        }
    }

    public class PropertyIsLessThanOrEqualToFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsLessThanOrEqualTo(expra, exprb);
        }
    }

    public class PropertyIsGreaterThanOrEqualToFactory : BinaryComparisonOpFactory
    {
        protected override BinaryComparisonOp CreateBinaryComparisonOp(IExpression expra, IExpression exprb)
        {
            return new PropertyIsGreaterThanOrEqualTo(expra, exprb);
        }
    }

    public class PropertyIsLikeFactory : IFilterFactory
    {
        #region IFilterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        #endregion
    }

    public class PropertyIsNullFactory : IFilterFactory
    {
        #region IFilterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new PropertyIsNull(xmlnode.SelectSingleNode("PropertyName").InnerText);
        }

        #endregion
    }

    public class PropertyIsBetweenFactory : IFilterFactory
    {
        #region IFilterFactory 成员

        public IFilter FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new PropertyIsBetween(builder.BuildExpression(xmlnode.SelectSingleNode("*[1]"))
                , builder.BuildExpression(xmlnode.SelectSingleNode("LowerBoundary"))
            , builder.BuildExpression(xmlnode.SelectSingleNode("UpperBoundary")));
        }

        #endregion
    }
    #endregion

    #region ExpressionFactory

    public abstract class BinaryOperatorFactory : IExpressionFactory
    {
        protected abstract IExpression CreateBinaryOperator(IExpression expra, IExpression exprb);

        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return CreateBinaryOperator(builder.BuildExpression(xmlnode.SelectSingleNode("*[1]"))
                , builder.BuildExpression(xmlnode.SelectSingleNode("*[2]")));
        }

        #endregion
    }

    public class AddFactory : BinaryOperatorFactory
    {
        protected override IExpression CreateBinaryOperator(IExpression expra, IExpression exprb)
        {
            return new Add(expra, exprb);
        }
    }

    public class SubFactory : BinaryOperatorFactory
    {
        protected override IExpression CreateBinaryOperator(IExpression expra, IExpression exprb)
        {
            return new Sub(expra, exprb);
        }
    }

    public class MulFactory : BinaryOperatorFactory
    {
        protected override IExpression CreateBinaryOperator(IExpression expra, IExpression exprb)
        {
            return new Mul(expra, exprb);
        }
    }

    public class DivFactory : BinaryOperatorFactory
    {
        protected override IExpression CreateBinaryOperator(IExpression expra, IExpression exprb)
        {
            return new Div(expra, exprb);
        }
    }

    public class PropertyNameFactory : IExpressionFactory
    {
        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new PropertyName(builder.BuildValue(xmlnode).ToString());
        }

        #endregion
    }

    public class LowerBoundaryFactory : IExpressionFactory
    {
        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new LowerBoundary(builder.BuildExpression(xmlnode.SelectSingleNode("*[1]")));
        }

        #endregion
    }

    public class UpperBoundaryFactory : IExpressionFactory
    {
        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new UpperBoundary(builder.BuildExpression(xmlnode.SelectSingleNode("*[1]")));
        }

        #endregion
    }

    public class LiteralFactory : IExpressionFactory
    {
        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            return new Literal(builder.BuildValue(xmlnode));
        }

        #endregion
    }

    public class FunctionFactory : IExpressionFactory
    {
        #region IExpressionFactory 成员

        public IExpression FromXmlNode(XmlNode xmlnode, FilterBuilder builder)
        {
            string name = xmlnode.SelectSingleNode("@name").InnerText;
            List<IExpression> args = new List<IExpression>();
            foreach (XmlNode i in xmlnode.SelectNodes("*"))
            {
                args.Add(builder.BuildExpression(i));
            }
            return new Function(name, args.ToArray());
        }

        #endregion
    }
    #endregion
}
