﻿using System;
using System.Collections.Generic;
using System.Text;
using SharpMap.Geometries;
using SharpMap.Data;
using SharpMap.Styles;
using System.Xml;
using Ogc.Filter;

namespace Ogc.Sld
{
    public class StyleRule
    {
        IStyle style;
        ITester tester;

        StyleRule elseRule;
        public StyleRule ElseRule
        {
            get { return elseRule; }
            set { elseRule = value; }
        }

        public static StyleRule[] FromSldFile(string sldfile, IWellKnownSymbolGetter symbolGetter)
        {
            XmlDocument doc = new XmlDocument();
            doc.Load(sldfile);
            StyleRule prev = null;
            List<StyleRule> ret = new List<StyleRule>();
            foreach (XmlNode i in doc.SelectNodes("//Rule"))
            {
                bool isElse = false;
                StyleRule r = BuildStyleRule(i, symbolGetter, out isElse);
                if (!isElse)
                    ret.Add(r);
                else
                {
                    if (prev == null)
                        throw new Exception("文件格式有误，ElseFilter之前必须有Filter。");
                    prev.elseRule = r;
                }
                prev = r;
            }
            return ret.ToArray();
        }

        private static StyleRule BuildStyleRule(XmlNode xmlnode, IWellKnownSymbolGetter symbolGetter, out bool isElse)
        {
            ITester tester = BuildFilter(xmlnode, out isElse);
            IStyle style = BuildStyle(xmlnode, symbolGetter);
            return new StyleRule(style, tester);
        }

        private static IStyle BuildStyle(XmlNode xmlnode, IWellKnownSymbolGetter symbolGetter)
        {
            StyleBuilder builder = new StyleBuilder(symbolGetter);

            XmlNode sn = xmlnode.SelectSingleNode("LineSymbolizer");
            if (sn != null)
                return builder.BuildLineSymbolStyle(sn);
            sn = xmlnode.SelectSingleNode("PointSymbolizer");
            if (sn != null)
                return builder.BuildPointSymbolStyle(sn);
            sn = xmlnode.SelectSingleNode("PolygonSymbolizer");
            if (sn != null)
                return builder.BuildPolygonSymbolStyle(sn);

            throw new NotImplementedException();
        }

        private static ITester BuildFilter(XmlNode xmlnode, out bool isElse)
        {
            isElse = false;
            到这里好像我的理解除了偏差，SLD其实定义的是图层绘制，使用了哪些Layer，使用了哪些Feature，使用了哪些Peoperty。
            并不是一个主题图那么简单。
            return null;
        }

        public StyleRule(IStyle style,ITester tester)
        {
            this.style = style;
            this.tester = tester;
        }

        public bool Test(SharpMap.Data.FeatureDataRow dr, out IStyle style)
        {
            if (tester == null || tester.Test(new FeatureDataRowGetter(dr)))
            {
                style = this.style;
                return true;
            }
            if (elseRule != null)
                return elseRule.Test(dr, out style);
            style = null;
            return false;
        }
    }

    class FeatureDataRowGetter : IPropertyGetter
    {
        FeatureDataRow dr;

        public FeatureDataRowGetter(FeatureDataRow dr)
        {
            this.dr = dr;
        }

        #region IPropertyGetter 成员

        public object GetProperty(string name)
        {
            return dr[name];
        }

        public Geometry GetGeometryProperty(string name)
        {
            return dr.Geometry;
        }

        #endregion
    }
}
