﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Ogc.Wms
{
    public class RequestGetMapParameters
    {
        Size mapSize;
        public Size MapSize
        {
            get { return mapSize; }
            set { mapSize = value; }
        }

        double[] bbox;
        public double[] BBox
        {
            get { return bbox; }
            set { bbox = value; }
        }

        string srs;
        public string SRS
        {
            get { return srs; }
            set { srs = value; }
        }

        string format;
        public string Format
        {
            get { return format; }
            set { format = value; }
        }

        List<KeyValuePair<string, string>> layers = new List<KeyValuePair<string, string>>();
        public void AddLayer(string layerName, string styleName)
        {
            layers.Add(new KeyValuePair<string, string>(layerName, styleName));
        }

        public void AddLayer(Layer layer)
        {
            foreach (Layer i in layer.Layers)
            {
                if (i.HasName)
                {
                    Style[] ss = i.Styles;
                    layers.Add(new KeyValuePair<string, string>(i.Name, ss.Length > 0 ? ss[0].Name : ""));
                }
                else
                    AddLayer(i);
            }
        }

        public string LayerNames
        {
            get
            {
                string ret = "";
                foreach (KeyValuePair<string, string> i in layers)
                {
                    ret += i.Key + ",";
                }
                return ret.TrimEnd(',');
            }
        }

        public string StyleNames
        {
            get
            {
                string ret = "";
                foreach (KeyValuePair<string, string> i in layers)
                {
                    ret += i.Value + ",";
                }
                return ret.TrimEnd(',');
            }
        }

        public string BBoxText
        {
            get { return bbox[0] + "," + bbox[1] + "," + bbox[2] + "," + bbox[3]; }
        }

        public override string ToString()
        {
            return "Layers=" + LayerNames
                + "&Styles=" + StyleNames
                + "&BBox=" + BBoxText
                + "&Width=" + mapSize.Width
                + "&Height=" + mapSize.Height
                + "&SRS=" + SRS
                + "&Format=" + Format;
        }

    }
}
