﻿using System;
using System.Data;
using System.Configuration;
using System.Web;
using System.IO;
using System.Text;
using System.Xml;
using Ogc.Wms;

/// <summary>
/// GetCapabilitiesProcessor 的摘要说明
/// </summary>
public class GetCapabilitiesProcessor
{
    HttpContext context;
    public GetCapabilitiesProcessor(HttpContext context)
    {
        this.context = context;
    }

    internal void Process()
    {
        XmlDocument doc = new XmlDocument();

        Capabilities caps = Capabilities.Create(doc);

        caps.CreateService();

        Capability cap = caps.CreateCapability();

        Request req = cap.CreateRequest();

        RequestItem reqGetCapabilities = req.AddItem("GetCapabilities");
        RequestItem reqGetMap = req.AddItem("GetMap");
        reqGetMap.AddFormat("image/png");

        Layer root = cap.CreateLayer();

        foreach (string i in File.ReadAllLines(Config.GetSridFile()))
        {
            root.AddSrs("EPSG:" + i.Substring(0, i.IndexOf(';')));
        }

        //从图层配置文件读取注册的图层
        foreach (string i in Config.GetWmsLayerFiles())
        {
            XmlDocument doc1 = new XmlDocument();
            doc1.Load(i);
            Layer l = root.AddLayer();
            l.Name = Path.GetFileNameWithoutExtension(i);
            l.Title = doc1.SelectSingleNode("Layer/Title").InnerText;
            l.Abstract = doc1.SelectSingleNode("Layer/Abstract").InnerText;
            foreach (XmlNode srs in doc1.SelectNodes("Layer/SRS"))
            {
                l.AddSrs(srs.InnerText);
            }
            foreach (XmlNode bbox in doc1.SelectNodes("Layer/BoundingBox"))
            {
                BoundingBox bb = l.AddBoundingBox();
                bb.SRS = bbox.SelectSingleNode("@SRS").Value;
                bb.MinX = double.Parse(bbox.SelectSingleNode("@minx").Value);
                bb.MinY = double.Parse(bbox.SelectSingleNode("@miny").Value);
                bb.MaxX = double.Parse(bbox.SelectSingleNode("@maxx").Value);
                bb.MaxY = double.Parse(bbox.SelectSingleNode("@maxy").Value);
            }
            LatLonBoundingBox llbb = l.CreateLatLonBoundingBox();
            llbb.MinX = double.Parse(doc1.SelectSingleNode("Layer/LatLonBoundingBox/@minx").Value);
            llbb.MinY = double.Parse(doc1.SelectSingleNode("Layer/LatLonBoundingBox/@miny").Value);
            llbb.MaxX = double.Parse(doc1.SelectSingleNode("Layer/LatLonBoundingBox/@maxx").Value);
            llbb.MaxY = double.Parse(doc1.SelectSingleNode("Layer/LatLonBoundingBox/@maxy").Value);
            foreach (XmlNode s in doc1.SelectNodes("Layer/Style"))
            {
                Style sld = l.AddStyle();
                XmlDocument doc2 = new XmlDocument();
                doc2.Load(Config.GetStyleFile(s.InnerText));
                sld.Name = s.InnerText;
                sld.Title = doc2.SelectSingleNode("Style/Title").InnerText;
                sld.Abstract = doc2.SelectSingleNode("Style/Abstract").InnerText;
            }
        }

        context.Response.ContentType = "file/xml";
        using(XmlWriter writer = XmlWriter.Create(context.Response.OutputStream))
            doc.WriteTo(writer);
    }

}
