﻿using System;
using System.Collections.Generic;
using System.Text;
using NUnit.Framework;
using Ogc.Filter;
using System.Xml;
using Ogc.Filter.Expressions;
using Ogc.Filter.ComparisonOps;

namespace unittests.Ogc.Filter
{
    [TestFixture]
    public class BuilderTests
    {
        [Test]
        public void BuildExpression()
        {
            FilterBuilder builder = new FilterBuilder();

            XmlDocument doc = new XmlDocument();
            doc.Load("Ogc/Filter/test_add.xml");

            IExpression expr = builder.BuildExpression(doc.DocumentElement);
            Assert.IsInstanceOf<Add>(expr);
            Assert.AreEqual(133, expr.Evaluate(null));
        }

        [Test]
        public void BuildFunction()
        {
            FilterBuilder builder = new FilterBuilder();

            XmlDocument doc = new XmlDocument();
            doc.Load("Ogc/Filter/test_func.xml");

            IExpression expr = builder.BuildExpression(doc.DocumentElement);
            Assert.IsInstanceOf<Function>(expr);
            Assert.AreEqual(Math.Sin(Math.Cos(90)), expr.Evaluate(null));
        }

        [Test]
        public void BuildFilter()
        {
            FilterBuilder builder = new FilterBuilder();

            XmlDocument doc = new XmlDocument();
            doc.Load("Ogc/Filter/test_filter1.xml");
            NamedGeomeryPropertyGetter getter = new NamedGeomeryPropertyGetter();
            getter.Add("PERSONS", 2100000d);
            getter.Add("MALES", 2000000d);

            IFilter f = builder.BuildFilter(doc.DocumentElement);
            Assert.IsInstanceOf<PropertyIsBetween>(f);
            Assert.IsFalse(f.Test(getter));
        }
    }
}
